SystemOrganization addCategory: #'FuelMetalevel-Clusters'!
SystemOrganization addCategory: #'FuelMetalevel-Mappers'!

FLMapper subclass: #FLClassPoolAssociationMapper
	instanceVariableNames: 'internalClasses classPoolBindings clustersByClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Mappers'!

!FLClassPoolAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:14' prior: 0!
I map variable class variable associations, i.e. bindings that belong to the class pool of a Class existent on Smalltalk globals. !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:31'!
accepts: anObject

	^ anObject isVariableBinding and: [ (self isClassVariableBinding: anObject) and: [ self isExternal: anObject ] ]! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/7/2011 16:39'!
clusterFor: anObject
	
	| theClass |
	theClass := classPoolBindings at: anObject.
	
	^clustersByClass 
		at: theClass 
		ifAbsentPut: [ FLClassPoolAssociationCluster for: theClass ]! !

!FLClassPoolAssociationMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/12/2011 11:07'!
initialize

	super initialize.
	clustersByClass := IdentityDictionary new.
	internalClasses := OrderedCollection new.
	self initializeClassPoolsBindings.! !

!FLClassPoolAssociationMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/12/2011 11:07'!
initializeClassPoolsBindings

	classPoolBindings := IdentityDictionary new.
	Smalltalk allClassesAndTraitsDo: [ :aClass | 
				aClass classPool associationsDo: [ :aBinding | 
					classPoolBindings at: aBinding put: aClass ] ].! !

!FLClassPoolAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:31'!
internalObjects: internalGlobalObjects

	internalClasses := internalGlobalObjects select: [ :anObject | anObject isBehavior ]! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isClassVariableBinding: anObject
	^classPoolBindings includesKey: anObject! !

!FLClassPoolAssociationMapper methodsFor: 'private' stamp: 'eem 6/8/2011 11:50'!
isExternal: anObject
	^ (internalClasses includes: (classPoolBindings at: anObject)) not! !

FLMapper subclass: #FLGlobalAssociationMapper
	instanceVariableNames: 'internalAssociations cluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Mappers'!

!FLGlobalAssociationMapper commentStamp: 'MartinDias 5/30/2011 01:15' prior: 0!
I map variable bindings that belong to Smalltalk global dictionary.

For example, 

	Smalltalk globals associationAt: #Transcript 
	
is mapped by me.!

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 02:01'!
accepts: anObject

	^ anObject isVariableBinding and: [(self isGlobalAssociation: anObject) and: [ self isExternalAssociation: anObject ]].
! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:34'!
clusterFor: anObject

	^cluster! !

!FLGlobalAssociationMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/11/2011 02:34'!
initialize

	super initialize.
	internalAssociations := OrderedCollection new.
	cluster := FLGlobalAssociationCluster new.! !

!FLGlobalAssociationMapper methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/6/2011 14:30'!
internalObjects: aCollection

	internalAssociations := aCollection collect: [:aGlobal | Smalltalk globals associationAt: aGlobal name ]! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:10'!
isExternalAssociation: anObject

	^ (internalAssociations includes: anObject ) not! !

!FLGlobalAssociationMapper methodsFor: 'private' stamp: 'MartinDias 5/20/2011 04:13'!
isGlobalAssociation: anObject

	^ Smalltalk globals includesAssociation: anObject! !

FLMapper subclass: #FLGlobalBehaviorMapper
	instanceVariableNames: 'internalGlobals globalBehaviors classSideCluster instanceSideCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Mappers'!

!FLGlobalBehaviorMapper commentStamp: 'MartinDias 8/6/2011 23:51' prior: 0!
I map classes that belong to Smalltalk global dictionary.

For example, Integer is mapped by me.!

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 23:56'!
accepts: anObject

	^(anObject isBehavior or: [ anObject isTrait ]) 
		and: [(self isGlobal: anObject) and: [self isExternal: anObject ]]! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:35'!
clusterFor: anObject

	^anObject isClassSide
		ifTrue: [ classSideCluster ]
		ifFalse: [ instanceSideCluster ]
! !

!FLGlobalBehaviorMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/12/2011 11:31'!
initialize

	super initialize.
	internalGlobals := OrderedCollection new.
	self initializeGlobalBehaviors.
	classSideCluster := FLGlobalClassSideCluster new.
	instanceSideCluster := FLGlobalClassCluster new.! !

!FLGlobalBehaviorMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/12/2011 10:22'!
initializeGlobalBehaviors
	globalBehaviors := IdentitySet new.
	Smalltalk globals allClassesAndTraitsDo: [:each | globalBehaviors add: each].
! !

!FLGlobalBehaviorMapper methodsFor: 'accessing' stamp: 'MartinDias 8/6/2011 20:34'!
internalObjects: anObject
	internalGlobals := anObject! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:28'!
isExternal: anObject

	^(internalGlobals includes: anObject instanceSide) not! !

!FLGlobalBehaviorMapper methodsFor: 'private' stamp: 'MartinDias 8/6/2011 23:31'!
isGlobal: anObject

	^globalBehaviors includes: anObject instanceSide! !

FLMapper subclass: #FLGlobalMapper
	instanceVariableNames: 'cluster'
	classVariableNames: 'Globals'
	poolDictionaries: ''
	category: 'FuelMetalevel-Mappers'!

!FLGlobalMapper commentStamp: 'MartinDias 8/18/2011 20:46' prior: 0!
I map global objects that belong to Smalltalk dictionary.

For example, Transcript is mapped by me.!

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MarianoMartinezPeck 9/28/2011 11:49'!
defaultGlobals

	^ IdentitySet new
		add: Display;
		add: World;
		add: Smalltalk;
		add: Transcript;
		add: SourceFiles;
		add: Processor;
		add: ImageImports;
		add: TextConstants;
		add: Undeclared;
		add: SystemOrganization;
		add: Sensor;
		add: ActiveWorld;
		add: ActiveHand;
		yourself! !

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MartinDias 8/18/2011 20:27'!
initialize

	self initializeGlobals ! !

!FLGlobalMapper classSide methodsFor: 'class initialization' stamp: 'MarianoMartinezPeck 9/28/2011 11:49'!
initializeGlobals

	Globals := self defaultGlobals.! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/18/2011 20:29'!
accepts: anObject

	^Globals includes: anObject! !

!FLGlobalMapper methodsFor: 'accessing' stamp: 'MartinDias 8/11/2011 02:36'!
clusterFor: anObject

	^cluster! !

!FLGlobalMapper methodsFor: 'initialize-release' stamp: 'MartinDias 12/6/2011 14:01'!
initialize

	super initialize.
	self flag: #todo. "This class side initialize is to solve some kind of weird bug when while serializing it cannot find the global ExpandedSourceFileArray(nil nil). I don't know how, but sometimes, depending on when the class side #initialize was sent, this global, SourcesFiles, can have 2 nil rather than the real value. Hence, we force a class side #initialize."
	self class initialize.
	cluster := FLGlobalValueCluster new.
		! !

!FLSerializer class methodsFor: '*FuelMetalevel-instance creation' stamp: 'MartinDias 9/25/2011 20:12'!
newFull

	| aSerializer |
	aSerializer := self new.
	aSerializer analyzer considerGlobalWhenPossible.
	^aSerializer! !

!FLAnalyzer methodsFor: '*FuelMetalevel' stamp: 'MarianoMartinezPeck 10/22/2011 12:53'!
allGlobalMappers

	^ (Array with: FLGlobalAssociationMapper with: FLClassPoolAssociationMapper with: FLGlobalMapper with: FLGlobalBehaviorMapper) 
		collect: [:aMapperClass | aMapperClass new internalObjects: internalClasses; yourself ]! !

!FLAnalyzer methodsFor: '*FuelMetalevel-configuring' stamp: 'MarianoMartinezPeck 12/2/2011 21:16'!
considerGlobalWhenPossible

	globalMappersFactory := [self allGlobalMappers].
	defaultMapperFactory := [FLFullDefaultMapper new].! !

!FLAnalyzer methodsFor: '*FuelMetalevel-configuring' stamp: 'MartinDias 9/11/2011 11:26'!
considerInternalAll: classes

	internalClasses addAll: classes.
	self considerGlobalWhenPossible.! !

FLIteratingCluster subclass: #FLBehaviorCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLBehaviorCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I have the common behavior for serializing class and trait behaviors in full detail.!

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
afterMaterializationStepWith: aMaterialization

	objects do:	[ :aClassOrTrait | 
		aClassOrTrait fuelInitializeTraitComposition.
		aClassOrTrait fuelAfterMaterialization ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstance: aClassOrTrait with: aMaterialization

	self subclassResponsibility! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstancesStepWith: aMaterialization

	objects do:	[ :aClassOrTrait | self materializePostInstance: aClassOrTrait with: aMaterialization ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:22'!
materializeReferencesOf: aClassOrTrait with: aMaterialization

	aClassOrTrait methodDictionary: aMaterialization nextEncodedReference.
	aClassOrTrait organization: aMaterialization nextEncodedReference.
	aClassOrTrait basicLocalSelectors: aMaterialization nextEncodedReference.
	aClassOrTrait traitComposition: aMaterialization nextEncodedReference.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesStepWith: aMaterialization

	objects do:	[ :aClassOrTrait | self materializeReferencesOf: aClassOrTrait with: aMaterialization ]! !

!FLBehaviorCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/30/2011 20:17'!
referencesOf: aClassOrTrait do: aBlock

	aBlock value: aClassOrTrait methodDictionary.
	aBlock value: aClassOrTrait organization.
	aBlock value: aClassOrTrait traitComposition.
	aBlock value: aClassOrTrait basicLocalSelectors.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstance: aClassOrTrait with: aSerialization

	self subclassResponsibility! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstancesStepWith: aSerialization  

	objects do:	[ :aClassOrTrait | self serializePostInstance: aClassOrTrait with: aSerialization ]! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesOf: aClassOrTrait with: aSerialization

	aSerialization encodeReferenceTo: aClassOrTrait methodDictionary.
	aSerialization encodeReferenceTo: aClassOrTrait organization. 
	aSerialization encodeReferenceTo: aClassOrTrait basicLocalSelectors.
	aSerialization encodeReferenceTo: aClassOrTrait traitComposition.! !

!FLBehaviorCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesStepWith: aSerialization

	objects do:	[ :aClassOrTrait | self serializeReferencesOf: aClassOrTrait with: aSerialization ]! !

FLBehaviorCluster subclass: #FLClassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLClassCluster commentStamp: 'MartinDias 7/30/2011 20:31' prior: 0!
I am a cluster for storing classes in full detail.!

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelInstanceSideBucket! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	| aMetaclass |
	aMetaclass := aMaterialization nextEncodedReference.
	^aMetaclass new! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstance: aClass with: aMaterialization

	| superclass category name instanceVariableNames format |
	superclass := aMaterialization nextEncodedReference.
	category := aMaterialization nextEncodedReference.
	name := aMaterialization nextEncodedReference.
	format := aMaterialization nextEncodedReference.
	instanceVariableNames := aMaterialization stream nextString subStrings: ' '.

	aClass 
		superclass: superclass;
		setFormat: format;
		setName: name;
		basicInstVarNames: instanceVariableNames;
		basicCategory: category; 
		methodDict: MethodDictionary new.
		! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeReferencesOf: aClass with: aMaterialization

	super materializeReferencesOf: aClass with: aMaterialization.

	aClass classPool: aMaterialization nextEncodedReference.
	aClass sharedPools: aMaterialization nextEncodedReference.
	aClass environment: aMaterialization nextEncodedReference.
	aClass basicSubclasses: aMaterialization nextEncodedReference.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aClass instVarAt: index put: aMaterialization nextEncodedReference ]! !

!FLClassCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/28/2011 19:41'!
referencesOf: aClass do: aBlock

	super referencesOf: aClass do: aBlock.

	aBlock value: aClass classSide.
	aBlock value: aClass superclass.
	aBlock value: aClass basicClassPool.
	aBlock value: aClass basicSharedPools. 
	aBlock value: aClass basicCategory.
	aBlock value: aClass name.
	aBlock value: aClass format.	
	aBlock value: aClass basicEnvironment.
	aBlock value: aClass basicSubclasses.
	
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aBlock value: (aClass instVarAt: index) ]! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: aClass with: aSerialization

	aSerialization encodeReferenceTo: aClass classSide! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstance: aClass with: aSerialization

	aSerialization encodeReferenceTo: aClass superclass.
	aSerialization encodeReferenceTo: aClass basicCategory.
	aSerialization encodeReferenceTo: aClass name.
	aSerialization encodeReferenceTo: aClass format.	
	aSerialization stream nextStringPut: aClass instanceVariablesString.
	self flag: #todo. "The array of variable names represents a circularity problem between clusters: if we encode the reference to such array, then the cluster for arrays should be before of Class cluster. But this would disable us to serialize an array of internal classes."
! !

!FLClassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeReferencesOf: aClass with: aSerialization

	super serializeReferencesOf: aClass with: aSerialization.

	aSerialization encodeReferenceTo: aClass basicClassPool. 
	aSerialization encodeReferenceTo: aClass basicSharedPools. 
	aSerialization encodeReferenceTo: aClass basicEnvironment.
	aSerialization encodeReferenceTo: aClass basicSubclasses.
 
	Class class instSize + 1 to: aClass class instSize 
		do: [ :index | aSerialization encodeReferenceTo: (aClass instVarAt: index) ]! !

FLBehaviorCluster subclass: #FLClassTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLClassTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing class traits in full detail.!

!FLClassTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelClassSideBucket! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^ClassTrait basicNew! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstance: aClassTrait with: aMaterialization

	aClassTrait 
		initialize.! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstances: aSerialization! !

!FLClassTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstance: aClassTrait with: aSerialization
! !

FLBehaviorCluster subclass: #FLMetaclassCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLMetaclassCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing metaclasses in full detail.!

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelClassSideBucket! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^Metaclass new! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstance: aMetaclass with: aMaterialization

	| superclass instanceVariableNames format |
	superclass := aMaterialization nextEncodedReference.
	instanceVariableNames := aMaterialization stream nextString subStrings: ' '.
	format := aMaterialization stream uint16.

	aMetaclass
		superclass: superclass 
		methodDictionary: MethodDictionary new 
		format: format;
		basicInstVarNames: instanceVariableNames.! !

!FLMetaclassCluster methodsFor: 'analyzing' stamp: 'MartinDias 7/29/2011 21:12'!
referencesOf: aMetaclass do: aBlock

	super referencesOf: aMetaclass do: aBlock.
	
	aBlock value: aMetaclass superclass.! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstances: aSerialization! !

!FLMetaclassCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstance: aMetaclass with: aSerialization

	aSerialization encodeReferenceTo: aMetaclass superclass.
	aSerialization stream nextStringPut: aMetaclass instanceVariablesString.
	aSerialization stream uint16: aMetaclass format.
	! !

FLBehaviorCluster subclass: #FLTraitCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLTraitCluster commentStamp: 'MartinDias 7/30/2011 20:30' prior: 0!
I am a cluster for storing traits in full detail.!

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/17/2011 00:27'!
clusterBucketIn: aClusterization

	^aClusterization metalevelInstanceSideBucket! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	^Trait basicNew! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializePostInstance: aTrait with: aMaterialization

	| classTrait category name environment |
	classTrait := aMaterialization nextEncodedReference.
	category := aMaterialization nextEncodedReference.
	name := aMaterialization nextEncodedReference.
	environment := aMaterialization nextEncodedReference.
	
	aTrait 
		initialize;
		name: name;
		basicCategory: category asSymbol;
		classTrait: classTrait;
		environment: environment.
	
	classTrait baseTrait: aTrait.! !

!FLTraitCluster methodsFor: 'analyzing' stamp: 'MartinDias 9/16/2011 23:48'!
referencesOf: aTrait do: aBlock

	super referencesOf: aTrait do: aBlock.

	aBlock value: aTrait classSide.
	aBlock value: aTrait basicEnvironment.
	aBlock value: aTrait basicCategory.
	aBlock value: aTrait name.
! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstances: aSerialization! !

!FLTraitCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializePostInstance: aTrait with: aSerialization

	aSerialization encodeReferenceTo: aTrait classSide.
	aSerialization encodeReferenceTo: aTrait basicCategory.
	aSerialization encodeReferenceTo: aTrait name.
	aSerialization encodeReferenceTo: aTrait basicEnvironment.
! !

FLIteratingCluster subclass: #FLClassPoolAssociationCluster
	instanceVariableNames: 'ownerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Clusters'!

!FLClassPoolAssociationCluster commentStamp: 'MartinDias 5/30/2011 01:17' prior: 0!
I am a cluster for class variable associations, i.e. bindings that belong to the class pool of a Class included on Smalltalk globals.!

!FLClassPoolAssociationCluster classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:35'!
for: aClass

	^ self basicNew
		initializeWith: aClass;
		yourself.! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/20/2011 22:24'!
clusterBucketIn: aClusterization

	^aClusterization baselevelBucket! !

!FLClassPoolAssociationCluster methodsFor: 'analyzing' stamp: 'MartinDias 6/5/2011 03:03'!
clusterReferencesDo: aBlock

	aBlock value: ownerClass! !

!FLClassPoolAssociationCluster methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/24/2011 16:39'!
initializeWith: aClass
	super initialize.
	ownerClass := aClass! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:23'!
materializeInstanceWith: aMaterialization

	| keyName |
	ownerClass := aMaterialization nextEncodedReference.
	keyName := aMaterialization stream nextString.
	^ ownerClass classPool associationAt: keyName asSymbol ! !

!FLClassPoolAssociationCluster methodsFor: 'printing' stamp: 'MarianoMartinezPeck 10/24/2011 10:57'!
printNameOn: aStream
	"Append to the argument, aStream, a sequence of characters that  
	identifies the receiver."
	
	super printNameOn: aStream.
	aStream nextPut: $[.
	ownerClass printOn: aStream.
	aStream nextPut: $].
	! !

!FLClassPoolAssociationCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 10/13/2011 16:20'!
serializeInstance: anAssociation with: aSerialization

	aSerialization encodeReferenceTo: ownerClass.
	aSerialization stream nextStringPut: anAssociation key! !

FLLightDefaultMapper subclass: #FLFullDefaultMapper
	instanceVariableNames: 'classCluster classTraitCluster metaclassCluster traitCluster namedCompiledMethodCluster'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelMetalevel-Mappers'!

!FLFullDefaultMapper commentStamp: 'MartinDias 8/7/2011 12:24' prior: 0!
I know how to map an object to its default cluster. I can map every object.!

!FLFullDefaultMapper methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 12/11/2011 00:03'!
initialize

	super initialize.
	classCluster := FLClassCluster new.
	classTraitCluster := FLClassTraitCluster new.
	metaclassCluster := FLMetaclassCluster new.
	traitCluster := FLTraitCluster new.
	namedCompiledMethodCluster := FLNamedPrimitiveMethodCluster withNoTrailerChange.
! !

!FLFullDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClass: aClass

	^classCluster! !

!FLFullDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:38'!
visitClassTrait: aClassTrait

	^classTraitCluster! !

!FLFullDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 12/2/2011 21:21'!
visitCompiledMethod: aCompiledMethod

	^aCompiledMethod isNamedPrimitive
		ifTrue:  [ namedCompiledMethodCluster ]
		ifFalse: [ compiledMethodCluster ]
! !

!FLFullDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:39'!
visitMetaclass: aMetaclass

	^metaclassCluster! !

!FLFullDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/11/2011 02:43'!
visitTrait: aTrait

	^traitCluster! !
FLGlobalMapper initialize!
